#!/usr/bin/env python
#
# Copyright (C) 2009 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Gnome application for controlling a random wallpaper via crontab.
"""

# Generate python libs
import sys, os
import gtk, gobject
import gtk.glade
import gettext, locale

sys.path.append('lib')

from crontab import CronTab
from gwc.config import config
from gwc.main import listers
from gwc.gtkviews import DropDown

APP        = 'wallchanger'
RAND_APP   = '/usr/bin/random-wallpaper'
PIXMAPS    = os.path.join('./', 'pixmaps')
GLADE_DIR  = os.path.abspath('glade')
#LOCALE_DIR = os.path.abspath('po')

#print "Looking for translations in %s" % LOCALE_DIR
#
#locale.setlocale(locale.LC_ALL)
#for module in gtk.glade, gettext:
#    module.bindtextdomain(APP, LOCALE_DIR)
#    module.textdomain(APP)
#gettext.install(APP)


class SettingsManager(object):
    """Manage the settings of this feature, save to crontab + xml config."""
    def __init__(self):
        self.init_gui()
        self.finish_gui()

    def glade_xml(self, file, inital):
        """Load a glade XML File"""
        return gtk.glade.XML(os.path.join(GLADE_DIR, file), inital, APP)

    def init_gui(self):
        """Initalise the gnome signals"""
        self.signals = {
            'destroyWindow'    : gtk.main_quit,
            'changeChange'     : self.change_change,
            'everyChanged'     : self.every_changed,
            'timeChanged'      : self.time_changed,
            'refreshWall'      : self.refresh_wall,
        }

        self.wtree  = self.glade_xml('wallchanger.glade', 'window')
        self.widget = self.wtree.get_widget
        self.window = self.widget('window')
        self.window.show()

        change = str(config['change'])
        if self.widget('change_'+change):
            self.widget('change_'+change).set_active(True)
            self.change_change(self.widget('change_'+change))

        every = config['everyunit']
        if not every:
            every = 0
        self.widget('crontype').set_active(int(every))

        time = config['everystep']
        if not time:
            time = 10
        self.widget('num').set_value(int(time))

        self.tlist = ListingTypes(self.widget('walls_from'),
            selected=self.change_type,
            unselected=None)
        for plugin in listers.values():
            self.tlist.add_item( plugin )

        # XXX Select type here

    def change_type(self, widget, text, deps):
        """Signal indicating a change in source image type"""
        if widget.get_active():
            config['type'] = str(text)
        for key in deps:
            if widget.get_active():
                self.widget(key).show()
            else:
                self.widget(key).hide()

    def change_change(self, widget):
        """Signal indicating a change in time type"""
        l = widget.get_label()
        if widget.get_active():
            if l[0] == 'N':
                config['change'] = 'never'
            elif l[0] == 'O':
                config['change'] = 'onboot'
            elif l[0] == 'E':
                config['change'] = 'every'
                self.widget('num').set_sensitive(True)
                self.widget('crontype').set_sensitive(True)
        else:
            if l[0] == 'E':
                self.widget('num').set_sensitive(False)
                self.widget('crontype').set_sensitive(False)
        self.update_cron()

    def every_changed(self, widget):
        """Signal for the every numberal box"""
        config['everyunit'] = widget.get_active()
        self.update_cron()

    def time_changed(self, widget):
        """Signal for the step widget"""
        config['everystep'] = int(widget.get_value())
        self.update_cron()

    def update_cron(self):
        """Update the saved crontab"""
        unit = config['everyunit']
        if not unit:
            unit = 0
        step = config['everystep']
        if not step:
            step = 10

        tab = CronTab()
        l = tab.find_command(RAND_APP.split('/')[-1])
        if l:
            e = l[0]
        else:
            e = tab.new(RAND_APP)

        e.command._command = RAND_APP
        e.special = None
        type = str(config['change'])
        if type == 'every':
            i = 0
            for s in e.slices:
                if i == int(unit):
                    s.every(step)
                elif i < int(unit):
                    s.parts = []
                    s.on(0)
                else:
                    s.every(1)
                i += 1
        elif type == 'onboot':
            e.every_reboot()
        else:
            tab.remove(e)
        tab.write()

    def finish_gui(self):
        """Finalisation of the gnome GUI"""
        self.wtree.signal_autoconnect(self.signals)

    def refresh_wall(self, widget):
        """Signal for loading a new random image for the background"""
        config.save()
        for line in os.popen(RAND_APP):
            print line


class ListingTypes(DropDown):
    """Operates the listing type combo box."""
    def setup(self):
        """Setup a treeview for showing services"""
        def text_cell_func(cell_layout, renderer, model, item_iter):
            """Render the text of the services tree view"""
            item = model.get_value(item_iter, 0)
            if item:
                renderer.set_property("markup", item.name)

        def icon_cell_func(column, cell, model, item_iter):
            """Reender the icons of the services tree view"""
            item = model.get_value(item_iter, 0)
            pixmap = None
            if item:
                file = os.path.join(PIXMAPS, item.icon)
                if os.path.exists(file):
                    pixmap = gtk.gdk.pixbuf_new_from_file(file)
                    cell.set_property("pixbuf", pixmap)
                else:
                    cell.set_property("stock-id", item.icon)
                    cell.set_property("stock-size", 24)
            cell.set_property("visible", True)

        svlist = super(ListingTypes, self).setup()
        # The icon
        renderer_icon = gtk.CellRendererPixbuf()
        renderer_icon.set_property("xpad", 8)
        renderer_icon.set_property("ypad", 2)
        svlist.pack_start(renderer_icon, False)
        svlist.set_cell_data_func(renderer_icon, icon_cell_func)
        # The name
        renderer = gtk.CellRendererText()
        svlist.pack_start(renderer, False)
        svlist.set_cell_data_func(renderer, text_cell_func)



if __name__ == "__main__":
    manager = SettingsManager()
    gtk.main()

